# -*- coding: utf-8 -*-

from __future__ import unicode_literals

__author__ = "Begon Jean-Michel <jm.begon@gmail.com>"



from datetime import datetime
try:
    # Python 2
    from urllib2 import URLError, HTTPError
    from urllib2 import urlopen
except ImportError:
    # Python 3+
    from urllib.error import URLError, HTTPError
    from urllib.request import urlopen
from bs4 import BeautifulSoup
import json
from abc import ABCMeta, abstractmethod
import re
try:
    # Python 2
    from HTMLParser import HTMLParser
except ImportError:
    # Python 3
    from html.parser import HTMLParser

from .event import Seminar
from .log import get_warn_log



__BASE_URL__ = "http://www.montefiore.ulg.ac.be"
__SEMINAR_SUFFIX__ = "/seminars"



class JSONParser(object):

    def _parse_date(self, date_as_text):
        return datetime.strptime(date_as_text, "%B %d, %Y (%A) %H:%M")

    def _remove_duplicate_eol(self, abstract):
        tmp = re.sub("( *(\r\n)+ *)+", "\r\n", abstract)
        return tmp.replace("\r\n", "\r\n\r\n")

    def _parse_contact(self, contact):
        return contact.replace("\\", "")

    def _html2str(self, dictionary):
        parser = HTMLParser()
        return {k:parser.unescape(v) for k,v in dictionary.items()}


    def __call__(self, jdict):
        if 'Seminar' not in jdict:
            get_warn_log(__name__).warn("Not a seminar: %s"%str(jdict))
        jdict = self._html2str(jdict)
        # Parse date
        date = self._parse_date(jdict["Time"])
        # Remove duplicate end of lines
        abstract = self._remove_duplicate_eol(jdict["Abstract"])
        # Parse contact
        contact = self._parse_contact(jdict["Contact"])
        return Seminar(name=jdict["Seminar"], speaker=jdict["Speaker"],
                       date=date, location=jdict["Location"],
                       contact=contact, abstract=abstract)




class DataSource(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def get_next_seminars(self):
        pass


    def __iter__(self):
        return iter(self.get_next_seminars())





class MontefioreGetter(DataSource):

    def __init__(self, parser=JSONParser(), base_url=__BASE_URL__,
                 seminar_suffix=__SEMINAR_SUFFIX__, fail_fast=False):
        self.base_url = base_url
        self.seminar_suffix = seminar_suffix
        self.parser = parser
        self.fail_fast = fail_fast


    def _get_link_list(self, page):
        soup = BeautifulSoup(page, 'html.parser')
        title_div = [x for x in soup.find_all("div")
                        if x.has_attr("seminar-title")]
        links = [x.find("a") for x in title_div]
        return [x.attrs["href"] for x in links if x is not None and
                            x.has_attr("href")]

    def _get_json(self, link):
        try:
            return json.loads(urlopen(self.base_url+link+"?json").read())
        except Exception as e:
            if self.fail_fast:
                raise
            errcls = e.__class__.__name__
            reason = e.message
            get_warn_log().warn("Could not load json at {link}. "
                "The reason is: {reason} ({errcls})".format(link=link,
                                                            reason=reason,
                                                            errcls=errcls))
            return None

    def _parse(self, jdict):
        try:
            return self.parser(jdict)
        except Exception as ex:
            if self.fail_fast:
                raise
            errcls = ex.__class__.__name__
            get_warn_log().warn("Could not parse event {jdict}. "
                "The reason is: {reason} ({errcls})".format(jdict=unicode(jdict),
                                                            reason=ex.message,
                                                            errcls=errcls))
            return None


    def get_next_seminars(self):
        url = self.base_url+self.seminar_suffix
        # No need to close: http://stackoverflow.com/questions/1522636/should-i-call-close-after-urllib-urlopen
        # No need to catch exception. In this case the program should crash
        page = urlopen(url).read()
        links = self._get_link_list(page)
        jsons = [self._get_json(link) for link in links]
        seminars = [self._parse(jdict) for jdict in jsons if jdict is not None]
        return [x for x in seminars if x is not None]







# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from nose.tools import assert_true, assert_false, assert_raises, assert_greater
from nose.tools import assert_less_equal

from datetime import datetime, timedelta

from montefevents import Seminar, Sender, Policy
from montefevents.notification import Channel
from montefevents.data import DataSource

def _monday12():
    return datetime(2016, 12, 12, 15, 10, 00)  # Monday


class NOPChannel(Channel):

    def __init__(self, debug=False):
        self.debug = debug

    def preview(self, event, decision):
        assert_true(self.debug)

    def send(self, event, decision):
        assert_false(self.debug)

class BuggyChannel(Channel):

    def __init__(self):
        self.err_count = 0

    def preview(self, event, decision):
        self.err_count += 1
        raise ValueError("BuggyChannel")

    def send(self, event, decision):
        return self.preview(event, decision)

class PseudoDataSource(DataSource):

    def get_next_seminars(self):
        deltas = range(-10, 10)
        seminars = []
        for delta in deltas:
            seminar = Seminar("test seminar", "test speaker",
                   _monday12() + timedelta(days=delta),
                   "test location", "test contact", "test abstract")
            seminars.append(seminar)
        return seminars


def test_sender():
    sender = Sender(Policy(_monday12()), NOPChannel())
    try:
        sender(PseudoDataSource())
        assert_true(True)
    except Exception as ex:
        assert_true(False, "Got exception {}".format(repr(ex)))

def test_debug():
    sender = Sender(Policy(_monday12()), NOPChannel(True), debug=True)
    sender(PseudoDataSource())

def test_fail_fast_false():
    bc = BuggyChannel()
    sender = Sender(Policy(_monday12()), bc, fail_fast=False)
    try:
        sender(PseudoDataSource())
        assert_true(True)
    except Exception as ex:
        assert_true(False, "Got exception {}".format(repr(ex)))
    assert_greater(bc.err_count, 1)

def test_fail_fast_true():
    bc = BuggyChannel()
    sender = Sender(Policy(_monday12()), bc, fail_fast=True)
    assert_raises(Exception, sender, PseudoDataSource())
    assert_less_equal(bc.err_count, 1)








# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from nose.tools import assert_equal

from datetime import datetime, timedelta

from montefevents import Seminar, Decision, Policy


def _today():
    return datetime.today()

def _day_after(date=_today()):
    return date + timedelta(days=1)

def _yesterday(date=_today()):
    return date - timedelta(days=-1)

def _last_week(date=_today()):
    return date - timedelta(days=-7)

def _day_after_day_after(date=_today()):
    return date + timedelta(days=2)

def _next_week(date=_today()):
    return date + timedelta(days=7)

def _much_later(date=_today()):
    return date + timedelta(days=30)


def _basic_seminar(date):
    return Seminar("test seminar", "test speaker",
                   date,
                   "test location", "test contact", "test abstract")

def _tba_seminar(date):
    return Seminar("TBA", "test speaker",
                   date,
                   "test location", "test contact", "test abstract")


def test_last_week():
    policy = Policy()
    sem = _basic_seminar(_last_week())
    tba = _tba_seminar(_last_week())
    assert_equal(policy(sem), Decision.IGNORE)
    assert_equal(policy(tba), Decision.IGNORE)


def test_yesterday():
    policy = Policy()
    sem = _basic_seminar(_yesterday())
    tba = _tba_seminar(_yesterday())
    assert_equal(policy(sem), Decision.IGNORE)
    assert_equal(policy(tba), Decision.IGNORE)



def test_today():
    policy = Policy()
    sem = _basic_seminar(_today())
    tba = _tba_seminar(_today())
    assert_equal(policy(sem), Decision.REMIND)
    assert_equal(policy(tba), Decision.IGNORE)



def test_day_after():
    ref_dates = [datetime(2016, 12, 5, 4, 10, 00),  # Monday
                 datetime(2016, 12, 6, 4, 10, 00)]  # Tuesday
    expectations = [Decision.ANNOUNCE, Decision.IGNORE]
    for ref_date, expectation in zip(ref_dates, expectations):
        policy = Policy(ref_date)
        seminar_date = _day_after(ref_date)
        sem = _basic_seminar(seminar_date)
        tba = _tba_seminar(seminar_date)
        assert_equal(sem.how_many_days_before(ref_date), 1)
        assert_equal(policy(sem), expectation)
        assert_equal(policy(tba), Decision.IGNORE)



def test_day_after_day_after():
    ref_dates = [datetime(2016, 12, 5, 4, 10, 00),  # Monday
                 datetime(2016, 12, 6, 4, 10, 00)]  # Tuesday
    expectations = [Decision.ANNOUNCE, Decision.IGNORE]
    for ref_date, expectation in zip(ref_dates, expectations):
        policy = Policy(ref_date)
        seminar_date = _day_after_day_after(ref_date)
        sem = _basic_seminar(seminar_date)
        tba = _tba_seminar(seminar_date)
        assert_equal(sem.how_many_days_before(ref_date), 2)
        assert_equal(policy(sem), expectation)
        assert_equal(policy(tba), Decision.IGNORE)


def test_next_week():
    policy = Policy()
    sem = _basic_seminar(_much_later())
    tba = _tba_seminar(_much_later())
    assert_equal(policy(sem), Decision.IGNORE)
    assert_equal(policy(tba), Decision.IGNORE)


def test_much_later():
    policy = Policy()
    sem = _basic_seminar(_much_later())
    tba = _tba_seminar(_much_later())
    assert_equal(policy(sem), Decision.IGNORE)
    assert_equal(policy(tba), Decision.IGNORE)







# -*- coding: utf-8 -*-

from __future__ import unicode_literals

__author__ = "Begon Jean-Michel <jm.begon@gmail.com>"


from abc import ABCMeta



class Event(object):
    __metaclass__ = ABCMeta

    def __init__(self, name, date):
        self.name = name
        self.date = date

    def get_date(self):
        return self.date

    def get_name(self):
        return self.name

    def how_many_days_before(self, date):
        return (self.date.date() - date.date()).days

    def date_as_str(self):
        return self.date.strftime("%A, %B %d, %H:%M")

    def is_filled(self):
        return self.name.strip().lower() != "tba"



# TODO mailto / site of contact
class Seminar(Event):
    """
    A seminar event.

    Constructor parameters
    ----------------------
    name : str
        The name of the event
    speaker : str
        Speaker's name
    date : `datetime`
        The date and the hour of the event
    location : str
        The location of the event
    contact : str
        A contact name
    abstract : str
        The abstract of the seminar
    """

    def __init__(self, name, speaker, date, location, contact, abstract):
        super(self.__class__, self).__init__(name, date)
        self.speaker = speaker
        self.location = location
        self.contact = contact
        self.abstract = abstract

    def __repr__(self):
        return "{cls}('{name}', '{speaker}', {date}, " \
               "'{location}', '{contact}', " \
               "'{abstract}')".format(cls=self.__class__.__name__,
                                    name=self.name,
                                    speaker=self.speaker, date=repr(self.date),
                                    location=self.location,
                                    contact=self.contact,
                                    abstract=self.abstract)
    def __str__(self):
        return "{cls}({name}, {speaker}, {date}, " \
               "{location}, {contact}".format(cls=self.__class__.__name__,
                                                name=self.name,
                                                speaker=self.speaker,
                                                date=str(self.date),
                                                location=self.location,
                                                contact=self.contact)












# -*- coding: utf-8 -*-

from __future__ import unicode_literals


__author__ = "Begon Jean-Michel <jm.begon@gmail.com>"

import sys
import logging


def get_warn_log(name=None):
    if name is None:
        name = __name__
    splits = name.split(".")
    return logging.getLogger(".".join([splits[0], "error"] + splits[1:]))

def get_info_log(name=None):
    if name is None:
        name = __name__
    splits = name.split(".")
    return logging.getLogger(".".join([splits[0], "info"] + splits[1:]))


def std_log():
    basename = __name__.split(".")[0]
    frmt = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    # Warn +
    warn_log = get_warn_log(basename)
    hdlr = logging.StreamHandler(sys.stderr)
    hdlr.setFormatter(frmt)
    warn_log.addHandler(hdlr)
    warn_log.setLevel(logging.WARN)
    # Info
    info_log = get_info_log(basename)
    hdlr = logging.StreamHandler(sys.stdout)
    hdlr.setFormatter(frmt)
    info_log.addHandler(hdlr)
    info_log.setLevel(logging.INFO)





# -*- coding: utf-8 -*-

from __future__ import unicode_literals

__author__ = "Begon Jean-Michel <jm.begon@gmail.com>"

from nose.tools import assert_equal, assert_true, assert_false
from nose.tools import assert_is_instance

from datetime import datetime

from montefevents import Seminar


def _basic_seminar():
    return Seminar("test seminar", "test speaker",
                   datetime(2016, 12, 6, 15, 10, 00),
                   "test location", "test contact", "test abstract")

def _tba_seminar():
    return Seminar("TBA", "test speaker",
                   datetime(2016, 12, 6, 15, 10, 00),
                   "test location", "test contact", "test abstract")


def test_create_seminar():
    try:
        _basic_seminar()
        _tba_seminar()
        assert_true(True)
    except:
        assert_true(False)

def test_repr():
    assert_is_instance(repr(_basic_seminar()), str)


def test_str():
    assert_is_instance(str(_basic_seminar()), str)

def test_dates():
    seminar = _basic_seminar()
    delta0 = datetime(2016, 12, 6, 10, 20, 00)
    delta7 = datetime(2016, 12, 13, 15, 20, 00)
    delta20 = datetime(2016, 12, 26, 15, 20, 00)
    delta_1 = datetime(2016, 12, 5, 15, 20, 00)
    assert_equal(seminar.how_many_days_before(delta0), 0)
    assert_equal(seminar.how_many_days_before(delta7), -7)
    assert_equal(seminar.how_many_days_before(delta20), -20)
    assert_equal(seminar.how_many_days_before(delta_1), +1)

def test_date_as_str():
    assert_equal(_basic_seminar().date_as_str(), 'Tuesday, December 06, 15:10')



def test_is_filled():
    sem = _basic_seminar()
    tba = _tba_seminar()
    assert_true(sem.is_filled())
    assert_false(tba.is_filled())










# -*- coding: utf-8 -*-

from __future__ import unicode_literals

__author__ = "Begon Jean-Michel <jm.begon@gmail.com>"

from datetime import datetime


class Decision(object):
    IGNORE = "IGNORE"
    ANNOUNCE = "ANNOUNCE"
    REMIND = "REMIND"

class Policy(object):
    def __init__(self, ref_date=datetime.today()):
        self.ref_date = ref_date

    def __call__(self, event):
        if not event.is_filled():
            return Decision.IGNORE
        n_days = event.how_many_days_before(self.ref_date)
        if 0 <= n_days < 7:
            if n_days == 0:
                return Decision.REMIND
            if self.ref_date.weekday() == 0:
                return Decision.ANNOUNCE
        return Decision.IGNORE



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

__author__ = "Begon Jean-Michel <jm.begon@gmail.com>"

from nose.tools import assert_is_instance

from email.mime.text import MIMEText

from datetime import datetime

from montefevents import Seminar, Decision, SMTPRenderer

def _basic_seminar():
    return Seminar("test seminar", "test speaker",
                   datetime(2016, 12, 6, 15, 10, 00),
                   "test location", "test contact", "test abstract")


def test_smtprenderer():
    rendrer = SMTPRenderer()
    mime = rendrer(_basic_seminar(), Decision.ANNOUNCE)
    assert_is_instance(mime, MIMEText)
    mime = rendrer(_basic_seminar(), Decision.REMIND)
    assert_is_instance(mime, MIMEText)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals, print_function

__author__ = "Begon Jean-Michel <jm.begon@gmail.com>"


from abc import ABCMeta, abstractmethod
import smtplib

from .policy import Decision
from .renderer import SMTPRenderer
from .log import get_warn_log, get_info_log


class Connection(object):
    """
    Note: coupling with the `with` statement, the connection object will
    be garbagged after the `with`
    """
    __metaclass__ = ABCMeta

    @abstractmethod
    def send(self, event):
        pass

    @abstractmethod
    def open(self):
        pass

    @abstractmethod
    def close(self):
        pass


    def __del__(self):
        self.close()



class SMTPConnection(Connection):

    def __init__(self, rendrer, smtp_serv, to_addrs, from_addr, port=25):
        self.rendrer = rendrer
        self.host = smtp_serv
        self.from_addr = from_addr
        self.to_addrs = to_addrs
        self.port = port
        self.conn = None

    def open(self):
        return self # Lazy open, will be done at send time

    def _open(self):
        self.conn = smtplib.SMTP(self.host, self.port)

    def close(self):
        if self.conn is not None:
            self.conn.quit()
        self.conn = None


    def send(self, event, decision):
        # Build email message
        msg = self.rendrer(event, decision)
        msg['From'] = self.from_addr
        msg['To'] = ", ".join(self.to_addrs)
        # Connect and send
        self._open()
        self.conn.sendmail(self.from_addr, self.to_addrs, msg.as_string())


class SMTPAuthConnection(SMTPConnection):

    def __init__(self, rendrer, smtp_serv, to_addrs, from_addr, username,
                 password, port=465):
        super(self.__class__, self).__init__(rendrer, smtp_serv, to_addrs,
                                             from_addr, port=port)
        self.username = username
        self.password = password

    def _open(self):
        self.conn = smtplib.SMTP(self.host, self.port)
        self.conn.starttls()
        self.conn.login(self.username, self.password)




class Channel(object):
    __metaclass__ = ABCMeta


    def __enter__(self):
        """
        Return an object with a `send` method
        """
        return self

    def __exit__(self, type, value, traceback):
        pass

    @abstractmethod
    def preview(self, event, decision):
        pass


class SMTPChannel(Channel):
    def __init__(self, renderer, smtp_serv, to_addrs, from_addr, username=None,
                 password=None, port=None):
        if not isinstance(renderer, SMTPRenderer):
            raise AttributeError("Renderer must be an instance of class '{}'. "
                                 "Given '{}".format(SMTPRenderer.__name__,
                                                    renderer.__class__.__name__))
        if username is None:
            username = from_addr
        if port is None:
            port = 25 if password is None else 465
        self.renderer = renderer
        self.smtp_serv = smtp_serv
        self.to_addrs = to_addrs
        self.from_addr = from_addr
        self.username = username
        self.password = password
        self.port = port

    def __enter__(self):
        if self.password is None:
            conn = SMTPConnection(self.renderer, self.smtp_serv, self.to_addrs,
                                  self.from_addr, port=self.port)
        else:
            conn = SMTPAuthConnection(self.renderer, self.smtp_serv,
                                      self.to_addrs, self.from_addr,
                                      self.username, self.password,
                                      port=self.port)
        conn.open()
        return conn

    def preview(self, event, decision):
        print("> FROM: ", self.from_addr)
        print("> TO:", ", ".join(self.to_addrs))
        print("> BY:", self.smtp_serv,":", self.port)
        if self.password is not None:
            print("> AUTH:", self.username)
        print("> CONTENT")
        self.renderer.preview(event, decision)





class Sender(object):

    def __init__(self, policy, channel, fail_fast=False,
                 debug=False):
        self.policy = policy
        self.channel = channel
        self.fail_fast = fail_fast
        self.debug = debug

    def __call__(self, datasource):
        warn_log = get_warn_log(__name__)
        info_log = get_info_log(__name__)
        for event in datasource:
            decision = self.policy(event)
            if decision == Decision.IGNORE:
                continue

            if self.debug:
                self.channel.preview(event, decision)
                continue

            try:
                with self.channel as connection:
                    connection.send(event, decision)
                    info_log.info("Event {} sent".format(event.name))

            except Exception as ex:
                if self.fail_fast:
                    raise
                warn_log.error("Could not send event {event}: "
                  "got exception {error}".format(event=event, error=repr(ex)))






# -*- coding: utf-8 -*-

from __future__ import unicode_literals

"""
The goal of this package is to provide automatic (email) notification of the
Montefiore unit's seminars.

Classes
=======
The package is organized around several classes. The main ones are:

`Seminar`
---------
The data structure representing a seminar.

`DataSource`
------------
A source from which the events can be gathered.

`Policy`
--------
The class responsible for deciding whether to notify of a given `Event`.

`Sender`
--------
The class which implements the overall logic: gathering, sorting and sending.

`Channel`
---------
The channel through which the notification will be sent.

`Renderer`
----------
A class which transform the event into the apropriate form to be able to go
through the channel.


Logging
=======
Normal information and warning/errors are subject to different loggers so that
they could be dispatched indenpendently.
Namely, information are logger of the form `basename.info*` whereas warning
are handled by logger of the form `basename.error`. The `log` module offers
two methods to handle the names. Respectively, `get_info_log` and `get_warn_log`.
This module also exposed `std_log` which redirect errrors to stderr and info
to stdout.
"""


__author__ = "Begon Jean-Michel <jm.begon@gmail.com>"
__version__ = '0.1.0'



from .event import Event, Seminar
from .policy import Decision, Policy
from .notification import Sender, SMTPChannel
from .data import MontefioreGetter
from .renderer import SMTPRenderer
from .log import get_warn_log, get_info_log, std_log


__all__ = ["Event", "Seminar", "Decision", "Policy", "Sender", "SMTPChannel",
           "MontefioreGetter", "SMTPRenderer", "get_warn_log", "get_info_log",
           "std_log"]

import logging
logging.getLogger(__name__).addHandler(logging.NullHandler())





# -*- coding: utf-8 -*-

#!/usr/bin/env python


from distutils.core import setup

import montefevents

NAME = 'montefevents'
VERSION = montefevents.__version__
AUTHOR = "Jean-Michel Begon"
AUTHOR_EMAIL = "jm.begon@gmail.com"
URL = 'https://github.com/jm-begon/montef-events'
DESCRIPTION = "Toolkit to provide automatic (email) notification of the Montefiore unit's seminars"
with open('README.md') as f:
    LONG_DESCRIPTION = f.read()
CLASSIFIERS = [
    'Development Status :: 4 - Beta',
    'Environment :: Console',
    'Intended Audience :: Developers',
    'Intended Audience :: Science/Research',
    'Intended Audience :: Education',
    'License :: OSI Approved :: BSD License',
    'Operating System :: OS Independent',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3.5',
    'Topic :: Utilities',
]

if __name__ == '__main__':
    setup(name=NAME,
          version=VERSION,
          author=AUTHOR,
          author_email=AUTHOR_EMAIL,
          url=URL,
          description=DESCRIPTION,
          long_description=LONG_DESCRIPTION,
          license='BSD3',
          classifiers=CLASSIFIERS,
          platforms='any',
          keywords='utility email notification',
          install_requires=["bs4"],
          packages=['montefevents', 'montefevents.test'],
          scripts=['bin/montefevents'])



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

__author__ = "Begon Jean-Michel <jm.begon@gmail.com>"


from nose.tools import assert_equal, assert_true

from datetime import datetime
import os


from montefevents import MontefioreGetter
from montefevents.data import JSONParser

#TODO More parsing case


jdict1 = {"Seminar":"Chance-Constrained Convex Mixed-Integer Programs and Beyond","Speaker":"Jesus De Loera (UC Davis)","Time":"December 09, 2016 (Friday) 05:00","Location":"Room R7, Montefiore Institute (B28)","Contact":"Quentin Louveaux (http:\/\/www.montefiore.ulg.ac.be\/~louveaux\/)","Abstract":"Chance-constrained convex mixed integer optimization, a branch of stochastic optimization, concerns convex mixed-integer programming problems in which constraints are imprecisely known but the problems need to be solved with a minimum probability of reliability or certainty. Such problems arise quite naturally in many areas of finance (e.g., portfolio planning where losses should not exceed some risk threshold), telecommunication (services agreements where contracts require network providers to guarantee with high probability that packet losses will not exceed a certain percentage), and facility location (for medical emergency response stations, while requiring high probability of coverage overall possible emergency scenarios). \r\n\r\n\r\n\r\n\r\n \r\nSuch problems are notoriously difficult to solve, because the feasible region is often not convex and the exact probabilities can be hard to compute exactly. We discuss a sampling approach that overcomes those obstacles. We have generalized a 2006 sampling algorithm of Calafiore-Campi’s for continuous variables that reduces the problem to solving deterministic convex mixed integer programming. A new generalization of Helly’s theorem is necessary to rigorously prove the sampling algorithms work as planned. \r\n\r\n\r\n \r\nIf time allows this talk will take a pick on the future of $S$-optimization theory, a powerful generalization of the traditional continuous, integer, and mixed-integer optimization, where now the variables take values in special proper subsets of $\\R^d$. All new theorems are joint work with R. La Haye, D. Oliveros, E. Roldan-Pensado."}
jdict2 = {"Seminar":"The next generation of artificial intelligence","Speaker":"Fran\u00e7ois Van Lishout (ULg - Montefiore)","Time":"November 25, 2016 (Friday) 22:15","Location":"Room R7, Montefiore Institute (B28)","Contact":"Fran\u00e7ois Van Lishout (f.vanlishout@ulg.ac.be)","Abstract":"World of Watson (WoW) is a 4-day conference organized every year in Las Vegas by IBM. I was invited to the 2016 edition by our partner NRB, which is itself an IBM-partner. \r\n\r\n\r\n\r\n\r\n\r\n\r\n\r\n\r\n\r\n\r\n\r\n\r\n\r\n\r\n\r\n\r\n\r\nThis year\u2019s WoW conference attracted around 17000 participants from all over the world, split into around 200 different rooms. I have put the focus mainly on talks linked with artifical intelligence, in the broad sense. I will try to present within an hour, as much as possible of the incredible things that I discovered there. \r\n\r\n\r\n\r\n\r\n\r\n\r\n\r\n\r\n\r\n\r\n\r\n\r\n\r\n\r\n\r\n\r\n\r\nTo already give you an idea of what my talk will be about, let me say a few words about Watson. It is an artificial intelligence cognitive computing system created by IBM, that can deal with massive amount of unstructured data and interface with human beings in ways that had never be seen before. It is well know from having won the jeopardy game-show in 2011, but this is not what it was about. It was about the beginning of the next era of computing. \r\n\r\n\r\n\r\n\r\n\r\n\r\n\r\n\r\n\r\n\r\n\r\n\r\n\r\n\r\n\r\n\r\n\r\nNowadays, Watson is used to improve decision making in health care, financial services and education, to name just three industries among many impacted by this technology. In this sense, Watson can be seen as an extended intelligence, rather than an artificial intelligence, since it\u2019s purpose is to augment human intelligence through the help of machines, rather than replacing humans through machines, …"}



def test_json_parse_date():
    expected = datetime(2016, 12, 9, 11, 0)
    given = JSONParser()._parse_date('December 09, 2016 (Friday) 11:00')
    delta = given - expected
    assert_equal(delta.total_seconds(), 0.0)

def test_remove_eol():
    original = "aaaa\r\n\r\n\r\n\r\n\r\n \r\n bbbb"
    expected = "aaaa\r\n\r\nbbbb"
    given = JSONParser()._remove_duplicate_eol(original)
    assert_equal(expected, given)


def test_weirdchars():
    parser = JSONParser()
    chars = [('Helly\u2019s theore', 'Helly’s theore')]
    for base, expected in chars:
        given = parser._html2str({0: base})
        assert_equal(given[0], expected)


def test_json_parsing():
    parser = JSONParser()
    jdicts = [jdict1, jdict2]
    names = ["Chance-Constrained Convex Mixed-Integer Programs and Beyond",
             "The next generation of artificial intelligence"]
    try:
        for jdict, name in zip(jdicts, names):
            seminar = parser(jdict)
            assert_equal(seminar.name, name)
        assert_true(True)
    except Exception as ex:
        assert_true(False, "Got {}".format(repr(ex)))


def test_montef_link_list():
    dir_path = os.path.dirname(os.path.realpath(__file__))
    seminar_suffix="seminars.html"
    path = os.path.join(dir_path, seminar_suffix)
    ds = MontefioreGetter()
    expected = ['http://www.montefiore.ulg.ac.be/seminar/chance-constrained-convex-mixed-integer-programs-and-beyond/', 'http://www.montefiore.ulg.ac.be/seminar/sound-fields-in-rooms-models-and-applications/']
    given = ds._get_link_list(open(path).read())
    assert_equal(given, expected)





# -*- coding: utf-8 -*-

from __future__ import unicode_literals, print_function

__author__ = "Begon Jean-Michel <jm.begon@gmail.com>"


from abc import ABCMeta, abstractmethod
from email.mime.text import MIMEText

from .policy import Decision


__CORE__ = """{header}
________________________________________________________________________________
{core}
________________________________________________________________________________

Looking forward to your active participation,

Regards,

The Montefiore Institute event organizers
"""

__HEADER_ANNOUNCE__ = """You are cordially invited to the following event organized by the Montefiore Institute.
More details can be found at http://www.montefiore.ulg.ac.be/seminars."""

__HEADER_REMIND__ = """A gentle reminder. More details can be found at http://www.montefiore.ulg.ac.be/seminars."""





class Renderer(object):
    __metaclass__ = ABCMeta


    @abstractmethod
    def preview(self, event, decision):
        pass

    @abstractmethod
    def render(self, event, decision):
        pass
    def __call__(self, event, decision):
        return self.render(event, decision)

class SMTPRenderer(Renderer):

    def _render_core(self, event):
        abstract = ""
        if len(event.abstract) > 50:
            abstract = "Abstract:\n{abstract}".format(abstract=event.abstract)
        return """Seminar: {title}

Speaker: {speaker}

Time: {time}

Location: {location}

Contact: {contact}

{abstract}""".format(title=event.name, speaker=event.speaker,
                     time=event.date_as_str(), location=event.location,
                     contact=event.contact, abstract=abstract)



    def _raw_render_announce(self, event):
        subject = "[All-montef] Seminar: {}".format(event.name)
        header = __HEADER_ANNOUNCE__
        core  = __CORE__.format(core=self._render_core(event), header=header)
        return subject, core

    def _raw_render_reminder(self, event):
        subject = "[All-montef] Reminder - Seminar: {}".format(event.name)
        header = __HEADER_REMIND__
        core  = __CORE__.format(core=self._render_core(event), header=header)
        return subject, core

    def get_raw(self, event, decision):
        if decision == Decision.ANNOUNCE:
            return self._raw_render_announce(event)
        if decision == Decision.REMIND:
            return self._raw_render_reminder(event)

    def render(self, event, decision):
        subject, content = self.get_raw(event, decision)
        msg = MIMEText(content, 'plain', 'utf-8')
        msg['Subject'] = subject
        return msg


    def preview(self, event, decision):
        subject, content = self.get_raw(event, decision)
        print(subject)
        print("")
        print(content)
        print("")
        print("")


